
package org.objectweb.fractal.bf.connectors.ws;

import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.bf.AbstractBindingFactoryPlugin;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.bf.CannotCreateSkeletonException;
import org.objectweb.fractal.bf.proxies.StubComponentFactory;
import org.objectweb.fractal.util.Fractal;

/**
 * A WS-Connector can export a Fractal interface to make it available via SOAP.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 *
 */
public class WsConnector extends AbstractBindingFactoryPlugin<WsExportId>
{
  
  Logger log = Logger.getLogger(WsConnector.class.getCanonicalName());
  
  /**
   * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createExportId(java.lang.String, org.objectweb.fractal.api.Component, java.util.Map)
   */
  public WsExportId createExportId(String itfName, Component owner,
      Map<String, String> arg2) throws BindingFactoryException
  {
    String ownerName=null;
    try
    {
      ownerName = Fractal.getNameController(owner).getFcName();
    }
    catch (NoSuchInterfaceException e)
    {
     throw new BindingFactoryException("Cannot get the name of the component owning the interface being exported with the SOAP-CXF connector",e);
    }
   return new WsExportId(itfName, ownerName);
  }

  /**
   * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createSkel(java.lang.String, java.lang.Object, org.objectweb.fractal.api.Component, java.util.Map)
   */
  public Component createSkel(String itfName, Object serverItf,
      Component component, Map<String, String> exportHints)
      throws CannotCreateSkeletonException
  {
    
    log.info("WsConnector - Creating skeleton..");
    
    String address = exportHints.get(WsConnectorConstants.ADDRESS);

    if (address == null)
    {
      address = WsConnectorConstants.DEFAULT_ADDRESS; 
    }

    Component skeleton = null;
    try
    {
      Factory adlFactory = FactoryFactory
          .getFactory(FactoryFactory.FRACTAL_BACKEND);

      skeleton = (Component) adlFactory.newComponent(
          "org.objectweb.fractal.bf.connectors.ws.WsServerBridge", null);
      
      // start the skeleton
      Fractal.getLifeCycleController(skeleton).startFc();
      
      WsServiceAttributes wsServiceAttributes = (WsServiceAttributes) skeleton
          .getFcInterface("serviceAttributes");
      
      InterfaceType it = (InterfaceType) ((Interface)serverItf).getFcItfType();
      Class<?> serviceClass = Class.forName(it.getFcItfSignature());
      
      wsServiceAttributes.configureWsWith(address, serviceClass);
     

    }
    catch (ADLException e)
    {
      throw new CannotCreateSkeletonException(
          "Some problem occured while  loading the ADL definition of the skeleton", e);
    }
    catch (NoSuchInterfaceException e)
    {
      throw new CannotCreateSkeletonException(
          "Some problem occured while loading the skeleton ", e);
    }
    catch (IllegalLifeCycleException e)
    {
     throw new CannotCreateSkeletonException(
          "Cannot start the skeleton component", e);
    }
    catch (ClassNotFoundException e)
    {
      throw new CannotCreateSkeletonException(
          "Cannot find class for the service", e);
    }

    return skeleton;
  }

  /**
   * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeSkeleton(org.objectweb.fractal.api.Component,
   *      java.util.Map)
   */
  public void finalizeSkeleton(Component skel,
      Map<String, String> finalizationHints) throws BindingFactoryException
  {

    log.info("Going to finalize SOAP/CXF skeleton");

    try
    {
      WsServiceAttributes wsServiceAttributes = (WsServiceAttributes) skel
          .getFcInterface("serviceAttributes");

      wsServiceAttributes.finalizeConfig(finalizationHints.get("itfName"));

      wsServiceAttributes.startWs();

    }
    catch (NoSuchInterfaceException e)
    {
      throw new BindingFactoryException(
          "Cannot find 'serviceAttributes' fcInterface on the specifies skeleton component",
          e);
    }

  }

  @Override
  public String toString()
  {
    return "SOAP/CXF Connector Plugin";
  }

  /**
   * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getStubComponentFactory()
   */
  public StubComponentFactory getStubComponentFactory()
  {
    return new WsConnectorStubComponentFactoryImpl();
  }

  /**
   * @throws BindingFactoryException
   * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getStubGenerationParameters(java.util.Map)
   */
  public String[] getStubGenerationParameters(Map<String, String> bindHints)
      throws BindingFactoryException
  {
    if (bindHints == null)
    {
      throw new BindingFactoryException("Binding hints were null");
    }
    String address = bindHints.get(WsConnectorConstants.ADDRESS);
    if (address == null)
    {
      throw new BindingFactoryException(
          "Missing parameter for Soap/CXF plugin (expected:"
              + WsConnectorConstants.ADDRESS + "");
    }
    String[] params = new String[]{address};
    return params;
  }

}
